-- Copyright (c) 2000 University of Utah and the Flux Group.
-- All rights reserved.
-- 
-- This file is part of the Knit component composition software.
-- 
-- Permission to use, copy, modify, distribute, and sell this software and
-- its documentation is hereby granted without fee, provided that the above
-- copyright notice and this permission/disclaimer notice is retained in all
-- copies or modified versions, and that both notices appear in supporting
-- documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
-- WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
-- THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-- PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
-- ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
-- 
-- Users are requested, but not required, to send to csl-dist@cs.utah.edu any
-- improvements that they make and grant Univ. of Utah redistribution rights.

module Main( main ) where

import MakeMakefile( mmf )
import System( getArgs )
import Monad( when )
import List( partition )

-- Typical usage ./Knit OSKITDIR=/z/reid/oskit BUILDDIR=/z/reid/build_oskit UNIT_PATH=~/Knit/units/oskit All.unit CatTest 

main = do
  args <- getArgs
  let (opts,args') = getOpts args
      (env,args'') = getEnv args'
  when (length args'' /= 2 || not (all (`elem` optList) opts)) $
    error "Usage: Knit [-f] [-c] [-X] {var=value} unit_file unit_name"
  let [fn,nm] = args''
      option x = x `elem` opts
  mmf fn nm env (option "-f") (option "-c") (option "-X")

getOpts :: [String] -> ([String],[String])
getOpts args = partition (\a -> take 1 a == "-") args

getEnv :: [String] -> ([(String,String)],[String])
getEnv args = (map split env,rest)
 where
  (env, rest) = partition ('=' `elem`) args
  split x = (pre, tail post) where (pre,post) = break (== '=') x

optList :: [String]
optList = 
  [ "-f"    -- flatten C code into one giant file
  , "-c"    -- check constraints
  , "-X"    -- don't output any files
  ]
