-- Copyright (c) 2000 University of Utah and the Flux Group.
-- All rights reserved.
-- 
-- This file is part of the Knit component composition software.
-- 
-- Permission to use, copy, modify, distribute, and sell this software and
-- its documentation is hereby granted without fee, provided that the above
-- copyright notice and this permission/disclaimer notice is retained in all
-- copies or modified versions, and that both notices appear in supporting
-- documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
-- WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
-- THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-- PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
-- ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
-- 
-- Users are requested, but not required, to send to csl-dist@cs.utah.edu any
-- improvements that they make and grant Univ. of Utah redistribution rights.

module PPUnits
	( defns          
	, defn       
	, unit        
	, import' 
	, export'
	, bundleType
	, btElement
	, binding     
	, unitInstance, posArgs, nameArgs, arg
        , constraint, property, propIdent, typeIdent
        , depend, objectSet, depRel
	, bundle, bundleElement  
        , flag, flags	
        , clause    
        , memberIdent
	, bundleIdent  
	, flagIdent   
	, unitIdent   
	, packageIdent   
	, bundleTIdent    
	, qualIdent       
	, ident       
	, relativeFileName   
	, fileName   
	) where

import AbstractUnits
import PrettyUtils

----------------------------------------------------------------
-- Pretty-printing Units
-- The names reflect their type. 
----------------------------------------------------------------

defns         :: [(Ident,Defn)]   -> Doc
defn          :: (Ident,Defn)     -> Doc
modunit       :: ModUnit          -> Doc
remunit       :: RemUnit          -> Doc
addunit       :: AddUnit          -> Doc
unit          :: Unit             -> Doc
rename        :: Rename           -> Doc
import'       :: Import           -> Doc
export'       :: Export           -> Doc
constraint    :: Constraint       -> Doc
property      :: Property         -> Doc
bundleType    :: BundleType       -> Doc
btElement     :: BTElement        -> Doc
binding       :: Binding          -> Doc
unitInstance  :: UnitInstance     -> Doc
posArgs       :: PosArgs          -> Doc
nameArgs      :: NameArgs         -> Doc
arg           :: Arg              -> Doc
bundle        :: Bundle           -> Doc
bundleElement :: BundleElement    -> Doc
depend        :: Depend           -> Doc
objectSet     :: ObjectSet        -> Doc
depRel        :: DependRelation   -> Doc
flags         :: Flags            -> Doc
flag          :: Flag             -> Doc
qualIdent     :: QualIdent        -> Doc
bundleIdent   :: BundleIdent      -> Doc
unitIdent     :: UnitIdent        -> Doc
bundleTIdent  :: BundleTIdent     -> Doc
packageIdent  :: PackageIdent     -> Doc
propIdent     :: PropIdent        -> Doc
ident         :: Ident            -> Doc
relativeFileName :: RelativeFileName -> Doc

defns = vmap defn

defn (x, BundleT s)  = text "bundletype"  <+> bundleTIdent x `eqn` bundleType s
defn (x, Unit    u)  = text "unit"        <+> unitIdent    x `eqn` unit       u
defn (x, ModUnit u)  = text "unit"        <+> unitIdent    x `eqn` modunit    u
defn (x, Flags   l)  = text "flags"       <+> flagIdent    x `eqn` flags      l
defn (x, Package us) = text "package"     <+> packageIdent x `eqn` hlist glue us
defn (x, Prop     )  = text "property"    <+> propIdent x
defn (x, Type supers) = text "type" <+> ident x <+> text "<=" <+> hlist typeIdent supers
defn (x, Rule cs  )  = text "rule"        <+> ruleIdent    x `eqn` hlist clause cs
defn (x, Doc s    )  = text "/*#" <+> text s <+> text "#*/"

modunit (x,rs,as)
  =  text "modify" <+> unitIdent x
  $$ text "deleting" <+> remunit rs
  $$ text "adding" <+> addunit as

remunit (is,es,cs,deps,gls,gs,defs,bs)
  =  text "imports" <+> brackets (hlist bundleIdent is) <> semi
  $$ text "exports" <+> brackets (hlist bundleIdent es) <> semi
  $$ vmap (\c -> text "constraints" <+> braces (constraint c) <> semi) cs
  $$ vmap (\d -> text "depends"     <+> braces (depend     d) <> semi) deps
  $$ text "globals"  <+> braces (hlist global gls)       <> semi
  $$ text "glue"     <+> braces (hlist glue gs)          <> semi
  $$ text "defaults" <+> braces (hlist bundleIdent defs) <> semi
  $$ text "link"     <+> braces (hlist bundleIdent bs)   <> semi

addunit (is,es,cs,deps,gls,gs,defs,bs)
  =  text "imports" <+> vlist2 lbrack comma rbrack (map import' is) <> semi
  $$ text "exports" <+> vlist2 lbrack comma rbrack (map export' es) <> semi
  $$ vmap (\c -> text "constraints" <+> braces (constraint c) <> semi) cs
  $$ vmap (\d -> text "depends"     <+> braces (depend     d) <> semi) deps
  $$ text "globals"  <+> braces (hlist global gls)       <> semi
  $$ text "glue"     <+> braces (hlist glue gs)          <> semi
  $$ text "defaults" <+> braces (hlist bundleIdent defs) <> semi
  $$ text "link"     <+> vlist2 lbrace semi rbrace (map binding bs)

unit (rs,ps,rules,cs,is,fs,ds,mbflatten,body) 
  =  text "imports" <+> vlist2 lbrack comma rbrack (map import' rs) <> semi
  $$ text "exports" <+> vlist2 lbrack comma rbrack (map export' ps) <> semi
  $$ vmap (\c -> text "rule" <+> braces (ruleIdent c) <> semi) rules
  $$ vmap (\c -> text "constraints" <+> braces (constraint c) <> semi) cs
  $$ vmap initializer is
  $$ vmap finalizer   fs
  $$ vmap (\d -> text "depends" <+> braces (depend d) <> semi) ds
  $$ maybe empty (\ flatten -> if flatten then text "flatten;" else text "noflatten;") mbflatten
  $$ either atomicUnit bindings body

initializer (x,ys) = text "initializer" <+> qualIdent x <+> text "=>" <+> objectSet ys <> semi
finalizer   (x,ys) = text "finalizer"   <+> qualIdent x <+> text "=>" <+> objectSet ys <> semi

atomicUnit (fs,rns) =  vmap fileWithFlags fs
                    $$ vmap rename rns

fileWithFlags (f, flags')
  =  file f
  $$ nest 2 (text "with flags" <+> flags flags' <> semi)
file (AnonCode c)
  =  text "%{" 
  $$ vmap text c
  $$ text "%}"
file f =
  text "files" <+> vlist2 lbrace comma rbrace (map relativeFileName [f])

bindings (gs,us,ds,bs) 
  =  text "global" <+> braces (hlist global gs)
  $$ text "glue" <+> braces (hlist glue us)
  $$ defaults ds
  $$ text "link"  <+> vlist2 lbrace semi rbrace (map binding  bs)

global = bundleIdent

defaults :: [BundleIdent] -> Doc
defaults [] = empty
defaults ds = text "defaults" <+> braces (hlist bundleIdent ds)

rename rn = text "rename" <+> braces (rename' rn) <> semi
 where
  rename' (RenameTo (x,y) z)
                    =   (bundleIdent x `dot` memberIdent y)
                    <+> text "to"
                    <+> ident z
  rename' (RenamePrefix x y) = bundleIdent x <+> text "with prefix" <+> ident y
  rename' (RenameSuffix x y) = bundleIdent x <+> text "with suffix" <+> ident y
  
import' (i, s) = bundleIdent i `hasType` bundleType s
export' = import'

bundleType (bt,fs) =  braces (hlist btElement bt) 
                   <> if null fs then empty else text " flags" <+> flags fs

btElement (Ident       x) = memberIdent x
btElement (Extends     s) = text "extends" <+> bundleTIdent s
btElement (Include     f) = text "include" <+> fileName f

binding (SingleBind x b) = bundleIdent x `eqn` bundle b
binding (MultiBind xs r) = hbracketList bundleIdent xs <+> text "<-" <+> unitInstance r

unitInstance (App u args)     = unitIdent u <+> text "<-" <+> either posArgs nameArgs args
unitInstance (RhsUnit u)      = text "unit" <+> braces (unit u)
unitInstance (RhsFlatten f u) = text (if f then "flatten" else "noflatten") <+> unitInstance u

posArgs = hbracketList arg

nameArgs (as,ellipsis) = braces $ fsep $ punctuate comma $ map nameArg as ++ if ellipsis then [text "..."] else []
 where
  nameArg :: (BundleIdent,Arg) -> Doc
  nameArg (x,i) = bundleIdent x `eqn` arg i

arg = either bundle (parens . unitInstance)

bundle (BundleIdent x)  = bundleIdent x
bundle (BundleBits  xs) = braces (hlist bundleElement xs)
bundle (BundleFind s)   = text "find" <+> bundleType s
bundle (BundleUnion xs) = foldr1 (\x y -> x <+> char '+' <+> y) (map bundle xs)

bundleElement (lhs,x) = memberIdent lhs `eqn` qualIdent x

constraint (Eq p1 p2) = property p1 <+> equals    <+> property p2
constraint (Le p1 p2) = property p1 <+> text "<=" <+> property p2

property (PropApp f xs) = propIdent f <> objectSet xs
property (PropConst c)  = typeIdent c

clause  :: Clause  -> Doc
formula :: Formula -> Doc
atomic  :: Atomic  -> Doc
aProp   :: AProp   -> Doc

clause (vs,Nothing,a) = text "var" <+> hlist ident vs <> semi <+> atomic a
clause (vs,Just f, a) = text "var" <+> hlist ident vs <> semi <+> formula f <+> text "=>" <+> atomic a

formula (Atomic a)  = atomic a
formula (f1 :|: f2) = formula f1 <+> text "|" <+> formula f2
formula (f1 :&: f2) = formula f1 <+> text "&" <+> formula f2
formula (Not f)     = text "!" <+> formula f
formula (Parens f)  = parens (formula f)

atomic (ALe x y)  = aProp x <+> text "<=" <+> aProp y
atomic (Import x) = text "import" <+> ident x
atomic (Export x) = text "export" <+> ident x
atomic (Init   x) = text "init"   <+> ident x
atomic (Fini   x) = text "fini"   <+> ident x

aProp (APropApp p x) = propIdent p <+> ident x
aProp (APropConst c) = typeIdent c

depend  (xs, r, ys) = objectSet xs <+> depRel r <+> objectSet ys

objectSet (ObjIdent i)     = bundleIdent i
objectSet ObjImports       = text "imports"
objectSet ObjExports       = text "exports"
objectSet ObjInits         = text "inits"
objectSet ObjFinis         = text "finis"
objectSet (ObjSet qs)      = braces (hlist qualIdent qs)
objectSet (ObjUnion d1 d2) = objectSet d1 <+> text "+" <+> objectSet d2
objectSet (ObjDiff  d1 d2) = objectSet d1 <+> text "-" <+> objectSet d2
objectSet (ObjParens d)    = parens (objectSet d)

depRel Calls    = text "needs"
depRel Precedes = text "<"

glue :: Glue -> Doc
glue (GlueIdent x)   = unitIdent x
glue (GluePackage x) = text "package" <+> packageIdent x

qualIdent (Just x,  y) = bundleIdent x `dot` memberIdent y
qualIdent (Nothing, y) =                     ident y

bundleIdent  = ident
unitIdent    = ident
bundleTIdent = ident
packageIdent = ident
flagIdent    = ident
propIdent    = ident
typeIdent    = ident
ruleIdent    = ident
memberIdent  = ident
ident        = text . fromIdent
relativeFileName = doubleQuotes . text . relativeName
fileName     = doubleQuotes . text

flags = vlist2 lbrace comma rbrace . map flag

flag (UseFlags name) = text "use" <+> flagIdent name
flag (RawFlag s) = text "raw" <+> (doubleQuotes . text) s

----------------------------------------------------------------
-- Utilities
----------------------------------------------------------------

hasType	:: Doc -> Doc -> Doc
hasType	l r = l <+> char ':' <+> r

dot :: Doc -> Doc -> Doc
dot l r = l <> char '.' <> r

eqn :: Doc -> Doc -> Doc
eqn l r = l <+> equals <+> r

eqn' :: Doc -> Doc -> Doc
eqn' lhs rhs 
  =  lhs <+> equals <+> lbrace
  $$ nest 2 rhs
  $$ rbrace

----------------------------------------------------------------
-- End 
----------------------------------------------------------------
