-- Copyright (c) 2000, 2001 University of Utah and the Flux Group.
-- All rights reserved.
-- 
-- This file is part of the Knit component composition software.
-- 
-- Permission to use, copy, modify, distribute, and sell this software and
-- its documentation is hereby granted without fee, provided that the above
-- copyright notice and this permission/disclaimer notice is retained in all
-- copies or modified versions, and that both notices appear in supporting
-- documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
-- WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
-- THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-- PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
-- ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
-- 
-- Users are requested, but not required, to send to csl-dist@cs.utah.edu any
-- improvements that they make and grant Univ. of Utah redistribution rights.

module Parser
	( Parser
	-- instance Functor (Parser s)
	-- instance Monad   (Parser s)
	-- instance MonadPlus (Parser s)
	, tokenP     -- :: (s -> Maybe (a,s)) -> Parser s a
	, runP       -- :: Parser s a -> s -> Maybe (a,s)
        , errorP     -- :: (s -> String) -> Parser s a
	, force      -- :: Parser s a -> Parser s a
	, zeroOrMore -- :: Parser s a -> Parser s [a]
	, oneOrMore  -- :: Parser s a -> Parser s [a]
	) where

import Monad
import Maybe

data Parser s a = P (s -> Maybe (a,s))

unP :: Parser s a -> (s -> Maybe (a,s))
unP (P a) = a

-- Access to state
tokenP  :: (s -> Maybe (a,s)) -> Parser s a

runP    :: Parser s a -> s -> Maybe (a,s)

tokenP get   = P $ get
runP m s     = (unP m) s

errorP :: (s -> String) -> Parser s a
errorP msg = P $ \s -> error (msg s)

instance Functor (Parser s) where
  fmap f xs = do{ x <- xs; return (f x) }

instance Monad (Parser s) where
  m >>= k  = P $ \ s -> do { (a,s') <- unP m s; unP (k a) s' }
  m >>  k  = P $ \ s -> do { (_,s') <- unP m s; unP k     s' }
  return a = P $ \ s -> return (a,s)

instance MonadPlus (Parser s) where
  mzero = P $ \ s -> mzero
  p `mplus` q = P $ \ s -> unP p s `mplus` unP q s

-- Wadler's force function
force             :: Parser s a -> Parser s a
force (P p)        = P $ \ s -> let x = p s in
				  Just (fst (fromJust x), snd (fromJust x))


zeroOrMore        :: Parser s a -> Parser s [a]
--zeroOrMore p       = force (oneOrMore p `mplus` return [])
zeroOrMore p       = oneOrMore p `mplus` return []

oneOrMore         :: Parser s a -> Parser s [a]
oneOrMore p        = do {x <- p; xs <- zeroOrMore p; return (x:xs)}

