-- Copyright (c) 2000 University of Utah and the Flux Group.
-- All rights reserved.
-- 
-- This file is part of the Knit component composition software.
-- 
-- Permission to use, copy, modify, distribute, and sell this software and
-- its documentation is hereby granted without fee, provided that the above
-- copyright notice and this permission/disclaimer notice is retained in all
-- copies or modified versions, and that both notices appear in supporting
-- documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
-- WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
-- THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-- PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
-- ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
-- 
-- Users are requested, but not required, to send to csl-dist@cs.utah.edu any
-- improvements that they make and grant Univ. of Utah redistribution rights.

module PrettyUtils
	( module PrettyUtils
	, module Pretty
	) where

import Pretty

vsep xs = xs `vsepdBy` blank

coco :: Doc
coco = text "::"

hsepdBy :: [Doc] -> Doc -> Doc
x `hsepdBy` punc = fsep (punctuate punc x)

vsepdBy :: [Doc] -> Doc -> Doc
x `vsepdBy` punc = vcat (punctuate punc x)

vlist1 :: Doc -> Doc -> Doc -> [Doc] -> Doc      
vlist1 l m r ds = l <+> vcat (punctuate m ds) <+> r

-- vlist2 l m r [d1, ... dn] 
--   =  l <+> d1
--   $$ m <+> d2
--   $$ m <+> dn
--   $$ r
vlist2 :: Doc -> Doc -> Doc -> [Doc] -> Doc      
vlist2 l m r []     = l <+> r
vlist2 l m r (d:ds) =  l <+> d 
                    $$ vmap (m <+>) ds
                    $$ r

hlist :: (a -> Doc) -> [a] -> Doc
hlist f = fsep . punctuate comma . map f

blank = text ""
hmap f = fsep . map f
vmap f = vcat . map f
htuple f = parens . hlist f
vtuple f = vlist2 lparen comma rparen . map f
vdecls f = vlist2 lbrace semi  rbrace . map f
vlist f = vcat . punctuate semi . map f
hbracketList f = brackets . hlist f
mb f = maybe empty f
comment x = text "/*" <+> x <+> text "*/"

binop op l r = l <+> op <+> r

showDoc doc = fullRender LeftMode 80 1.5 my_string_txt "" doc

my_string_txt (Chr c)   s  = c:s
my_string_txt (Str s1)  s2 = s1 ++ s2
my_string_txt (PStr s1) s2 = s1 ++ s2

-- class PP a where pp :: a -> Doc

--instance Show a => PP a where pp = text.show