-- Copyright (c) 2000, 2001 University of Utah and the Flux Group.
-- All rights reserved.
-- 
-- This file is part of the Knit component composition software.
-- 
-- Permission to use, copy, modify, distribute, and sell this software and
-- its documentation is hereby granted without fee, provided that the above
-- copyright notice and this permission/disclaimer notice is retained in all
-- copies or modified versions, and that both notices appear in supporting
-- documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
-- WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
-- THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-- PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
-- ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
-- 
-- Users are requested, but not required, to send to csl-dist@cs.utah.edu any
-- improvements that they make and grant Univ. of Utah redistribution rights.

module TopSort
        ( topSortR
        , Tree(..), Forest
        , tree
        , forest
        , postorderF
        , postorderT
        ) where

import Maybe( fromMaybe )

----------------------------------------------------------------
-- Graph algorithms
----------------------------------------------------------------

-- reverse topological sort (returns postorder of depth first forest)
topSortR :: Eq a => [(a,[a])] -> [a] -> [a]
topSortR rel as = postorderF $ fst $ forest rel [] as

data Tree a = Node a (Forest a)
type Forest a = [Tree a]

tree :: Eq a => [(a,[a])] -> [a] -> a -> (Tree a, [a])
tree rel seen a = (Node a f, seen1)
 where
  (f,seen1) = forest rel (a:seen) (fromMaybe [] (lookup a rel))

forest :: Eq a => [(a,[a])] -> [a] -> [a] -> (Forest a, [a])
forest rel seen [] = ([], seen)
forest rel seen (a:as) 
  | a `elem` seen
  = forest rel seen as
  | otherwise
  = (t:f, seen2)
 where
  (t, seen1) = tree rel seen a
  (f, seen2) = forest rel seen1 as

postorderF :: Forest a -> [a]
postorderF ts = concatMap postorderT ts

postorderT :: Tree a -> [a]
postorderT (Node a ts) = postorderF ts ++ [a]

----------------------------------------------------------------
-- End
----------------------------------------------------------------