-- Copyright (c) 2000 University of Utah and the Flux Group.
-- All rights reserved.
-- 
-- This file is part of the Knit component composition software.
-- 
-- Permission to use, copy, modify, distribute, and sell this software and
-- its documentation is hereby granted without fee, provided that the above
-- copyright notice and this permission/disclaimer notice is retained in all
-- copies or modified versions, and that both notices appear in supporting
-- documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
-- WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
-- THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-- PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
-- ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
-- 
-- Users are requested, but not required, to send to csl-dist@cs.utah.edu any
-- improvements that they make and grant Univ. of Utah redistribution rights.

module ZEncode
	( zEncode
	, zDecode
        , xEncode
	) where

import Char( isAlphaNum, ord, chr, isDigit )

----------------------------------------------------------------
-- Z encoding
--
-- This code is based on a hazy memory of a similar function inside ghc.
-- The idea is to turn an arbitrary (non-empty) string into a valid C
-- identifier by using Z as an escape character.
----------------------------------------------------------------

-- use this to generate valid C function names
zEncode :: String -> String
zEncode [] = []
zEncode ('Z':xs) = 'Z':'Z': zEncode xs
zEncode (x:xs)
  | isAlphaNum x
  = x : zEncode xs
  | otherwise
  = 'Z' : show (ord x) ++ zEncode xs

zDecode :: String -> String
zDecode [] = []
zDecode ('Z':'Z':xs) = 'Z' : zDecode xs
zDecode ('Z':xs)     = chr (read x) : zDecode y
 where 
  (x,y) = span isDigit xs
zDecode (x:xs)
  | isAlphaNum x       -- pure paranoia
  = x : zDecode xs

-- use this for filenames
xEncode :: String -> String
xEncode [] = []
xEncode ('X':xs) = 'X':'X': xEncode xs
xEncode (x:xs)
  | isAlphaNum x || x `elem` "._"
  = x : xEncode xs
  | x == '/'    -- hope for no collisions
  = '_' : xEncode xs
  | otherwise
  = 'X' : show (ord x) ++ xEncode xs

