/*
 * Copyright (c) 2000 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Knit component composition software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

/* Simple graph algorithms 
 */

typedef int data;
#define baddata (-1)  // shouldn't look like an index into a matrix!

typedef struct vector* vector;
typedef struct matrix* matrix;
typedef struct tree*   tree;
typedef struct forest* forest;

extern void addRefV(vector v);
extern void freeV(vector v);
extern vector emptyV(void);
extern void setV(vector v, int i, data d);
extern data getV(vector v, int i);

/* search unsorted vector for element */
extern int elemV(vector v, data d);

/* insert element at end of vector */
extern void appendV(vector v, data d);

extern void   reverseV(vector v);
extern int    sizeV(vector v);
extern vector intersectV(vector x, vector y);

extern void   freeM(matrix m);
extern void   freeM_but_not_vectors(matrix m);
extern matrix emptyM(void);
extern void   setM(matrix m, int i, vector v);
extern vector getM(matrix m, int i);
extern void   appendM(matrix m, vector v);
extern data   get(matrix m, int i, int j);
extern void   set(matrix m, int i, int j, data d);
extern void   add(matrix m, int i, int j);
extern void   printV(vector v);
extern void   printM(matrix m);
extern void   freeT(tree t);
extern void   freeF(forest f);
extern forest emptyF(void);
extern tree   mkT(data d, forest f);
extern int    isSingletonT(tree t);
extern int    isEmptyF(forest f);
extern void   appendF(forest f, tree t);
extern void   printT(tree t);
extern void   printF(forest f);

/*
 * reachableBFS
 *
 * Returns list of elements reachable from v using m as an
 * adjacency matrix (ie assuming that m[i] is a list of 
 * things reachable from i).
 *
 * Uses breadth-first search
 */
extern vector reachableBFS(matrix m, vector v);

/*
 * reachableDFS
 *
 * Find list of elements reachable from v using m as an
 * adjacency matrix (ie assuming that m[i] is a list of 
 * things reachable from i).  Updates vector r in place.
 *
 * Uses depth-first search
 */
extern void reachableDFS(matrix m, vector r, vector v);

/*
 * Depth First Forest
 */
extern forest dfs(matrix m, vector v);
extern void reverseF(forest f);
extern void postorderT(vector v, tree t);
extern void postorderF(vector v, forest f);

/*
 * topSort
 *
 * Sort list v according to dependency matrix m (ie assuming that m[i]
 * is a list of things greater than i and contains no duplicates).
 */
extern vector topSort(matrix m, vector v);

/*
 * topSort2
 *
 * if acyclic, returns topological sort
 * if cyclic, returns first cycle
 */
vector topSort2(matrix m, vector v, vector interesting, int *cyclic);

