-- Copyright (c) 2000 University of Utah and the Flux Group.
-- All rights reserved.
-- 
-- This file is part of the Knit component composition software.
-- 
-- Permission to use, copy, modify, distribute, and sell this software and
-- its documentation is hereby granted without fee, provided that the above
-- copyright notice and this permission/disclaimer notice is retained in all
-- copies or modified versions, and that both notices appear in supporting
-- documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
-- WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
-- THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-- PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
-- ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
-- 
-- Users are requested, but not required, to send to csl-dist@cs.utah.edu any
-- improvements that they make and grant Univ. of Utah redistribution rights.

module Graph
        ( module Graph
        ) where

import StdDIS
import Monad( zipWithM_ )



data Graph = Graph ForeignObj
data List  = List  ForeignObj
type Data  = Int

emptyGraph :: IO Graph
emptyGraph =
  prim_emptyGraph >>= \ (gc_res3,gc_res1) ->
  (makeForeignObj gc_res1 gc_res3) >>= \ gc_res2 ->
  (return ((Graph gc_res2)))
primitive prim_emptyGraph :: IO (Addr,Addr)

addEdge :: Graph -> Data -> Data -> IO ()
addEdge gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Graph arg1) ->
  prim_addEdge arg1 arg2 arg3}
primitive prim_addEdge :: ForeignObj -> Int -> Int -> IO ()

emptyList :: IO List
emptyList =
  prim_emptyList >>= \ (gc_res3,gc_res1) ->
  (makeForeignObj gc_res1 gc_res3) >>= \ gc_res2 ->
  (return ((List gc_res2)))
primitive prim_emptyList :: IO (Addr,Addr)

size :: List -> IO Int
size gc_arg1 =
  case gc_arg1 of { (List arg1) ->
  prim_size arg1 >>= \ (res1) ->
  (return (res1))}
primitive prim_size :: ForeignObj -> IO (Int)

get :: List -> Int -> IO Data
get gc_arg1 arg2 =
  case gc_arg1 of { (List arg1) ->
  prim_get arg1 arg2 >>= \ (res1) ->
  (return (res1))}
primitive prim_get :: ForeignObj -> Int -> IO (Int)

fromList :: List -> IO [Data]
fromList xs = do
  s <- size xs
  mapM (get xs) [0..s-1]

insert :: List -> Data -> IO ()
insert gc_arg1 arg2 =
  case gc_arg1 of { (List arg1) ->
  prim_insert arg1 arg2}
primitive prim_insert :: ForeignObj -> Int -> IO ()

reachableBFS :: Graph -> List -> List
reachableBFS gc_arg1 gc_arg2 =
  unsafePerformIO(
    case gc_arg1 of { (Graph arg1) ->
    case gc_arg2 of { (List arg2) ->
    prim_reachableBFS arg1 arg2 >>= \ (gc_res3,gc_res1) ->
    (makeForeignObj gc_res1 gc_res3) >>= \ gc_res2 ->
    (return ((List gc_res2)))}})
primitive prim_reachableBFS :: ForeignObj -> ForeignObj -> IO (Addr,Addr)
topSort :: Graph -> List -> List
topSort gc_arg1 gc_arg2 =
  unsafePerformIO(
    case gc_arg1 of { (Graph arg1) ->
    case gc_arg2 of { (List arg2) ->
    prim_topSort arg1 arg2 >>= \ (gc_res3,gc_res1) ->
    (makeForeignObj gc_res1 gc_res3) >>= \ gc_res2 ->
    (return ((List gc_res2)))}})
primitive prim_topSort :: ForeignObj -> ForeignObj -> IO (Addr,Addr)

topSort2 :: Graph -> List -> List -> (Bool,List)
topSort2 gc_arg1 gc_arg2 gc_arg3 =
  unsafePerformIO(
    case gc_arg1 of { (Graph arg1) ->
    case gc_arg2 of { (List arg2) ->
    case gc_arg3 of { (List arg3) ->
    prim_topSort2 arg1 arg2 arg3 >>= \ (res1,gc_res4,gc_res2) ->
    (unmarshall_bool_ res1) >>= \ gc_res1 ->
    (makeForeignObj gc_res2 gc_res4) >>= \ gc_res3 ->
    (return ((gc_res1,(List gc_res3))))}}})
primitive prim_topSort2 :: ForeignObj -> ForeignObj -> ForeignObj -> IO (Int,Addr,Addr)


needPrims_hugs 2
