D			[0-9]
L			[a-zA-Z_]
H			[a-fA-F0-9]
E			[Ee][+-]?{D}+
FS			(f|F|l|L)
IS			(u|U|l|L)*

%{
/*
 * Copyright (c) 2000 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Knit component composition software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
/* 
 * This is based on Jeff Lee's grammar and lexer for the April 1985
 * draft of the ANSI C standard as posted to net.sources by
 * tps@sdchem.UUCP (Tom Stockfisch) on 3 Mar 87.  
 * The original README is in gram.README.
 *
 * This has since been heavily modified to handle typedefs
 * differently, to support features of the current ANSI standard, to
 * support many gcc extensions and to generate an "sexpr" tree for
 * communicating with Knit.  
 */

#include <stdio.h>
#include "sexpr.h"
#define YYSTYPE sexpr
#include "y.tab.h"

YYSTYPE yylval;

void count();
void comment();
int check_type(void);
%}

%%
"/*"			{ comment(); }

"auto"			{ count(); return(AUTO); }
"break"			{ count(); return(BREAK); }
"case"			{ count(); return(CASE); }
"char"			{ count(); return(CHAR); }
"const"			{ count(); return(CONST); }
"__const__"		{ count(); return(CONST); }
"__const"		{ count(); return(CONST); }
"continue"		{ count(); return(CONTINUE); }
"default"		{ count(); return(DEFAULT); }
"do"			{ count(); return(DO); }
"double"		{ count(); return(DOUBLE); }
"else"			{ count(); return(ELSE); }
"enum"			{ count(); return(ENUM); }
"extern"		{ count(); return(EXTERN); }
"float"			{ count(); return(FLOAT); }
"for"			{ count(); return(FOR); }
"goto"			{ count(); return(GOTO); }
"if"			{ count(); return(IF); }
"int"			{ count(); return(INT); }
"long"			{ count(); return(LONG); }
"register"		{ count(); return(REGISTER); }
"return"		{ count(); return(RETURN); }
"short"			{ count(); return(SHORT); }
"signed"		{ count(); return(SIGNED); }
"__signed__"		{ count(); return(SIGNED); }
"sizeof"		{ count(); return(SIZEOF); }
"static"		{ count(); return(STATIC); }
"struct"		{ count(); return(STRUCT); }
"switch"		{ count(); return(SWITCH); }
"typedef"		{ count(); return(TYPEDEF); }
"union"			{ count(); return(UNION); }
"unsigned"		{ count(); return(UNSIGNED); }
"void"			{ count(); return(VOID); }
"volatile"		{ count(); return(VOLATILE); }
"__volatile__"		{ count(); return(VOLATILE); }
"while"			{ count(); return(WHILE); }
"__typeof__"		{ count(); return(TYPEOF); }
"__inline"		{ count(); return(INLINE); }
"__inline__"		{ count(); return(INLINE); }
"__attribute__"		{ count(); return(ATTRIBUTE); }
"__label__"             { count(); return(LABEL); }
"inline"		{ count(); return(INLINE); }
"attribute"		{ count(); return(ATTRIBUTE); }
"__extension__"		{ count(); return(EXTENSION); }
"asm"                   { count(); return(ASM); }
"__asm"                 { count(); return(ASM); }
"__asm__"               { count(); return(ASM); }

"__FUNCTION__"		{ count(); 
                          yylval = string("__FUNCTION__"); 
                          return(STRING_LITERAL); 
                        }
"__PRETTY_FUNCTION__"	{ count(); 
                          yylval = string("__PRETTY_FUNCTION__"); 
                          return(STRING_LITERAL); 
                        }

{L}({L}|{D})*		{ count(); yylval = other(yytext); return(check_type()); }

0[xX]{H}+{IS}?		{ count(); return(CONSTANT); }
0{D}+{IS}?		{ count(); return(CONSTANT); }
{D}+{IS}?		{ count(); return(CONSTANT); }
'(\\.|[^\\'])+'		{ count(); return(CONSTANT); }

{D}+{E}{FS}?		{ count(); return(CONSTANT); }
{D}*"."{D}+({E})?{FS}?	{ count(); return(CONSTANT); }
{D}+"."{D}*({E})?{FS}?	{ count(); return(CONSTANT); }

\"(\\.|[^\\"])*\"	{ count(); 
                          yytext[yyleng-1] = 0; // ignore the quotes
                          yylval = string(yytext+1); 
                          return(STRING_LITERAL); 
                        }
">>="			{ count(); return(RIGHT_ASSIGN); }
"<<="			{ count(); return(LEFT_ASSIGN); }
"+="			{ count(); return(ADD_ASSIGN); }
"-="			{ count(); return(SUB_ASSIGN); }
"*="			{ count(); return(MUL_ASSIGN); }
"/="			{ count(); return(DIV_ASSIGN); }
"%="			{ count(); return(MOD_ASSIGN); }
"&="			{ count(); return(AND_ASSIGN); }
"^="			{ count(); return(XOR_ASSIGN); }
"|="			{ count(); return(OR_ASSIGN); }
">>"			{ count(); return(RIGHT_OP); }
"<<"			{ count(); return(LEFT_OP); }
"++"			{ count(); return(INC_OP); }
"--"			{ count(); return(DEC_OP); }
"->"			{ count(); return(PTR_OP); }
"&&"			{ count(); return(AND_OP); }
"||"			{ count(); return(OR_OP); }
"<="			{ count(); return(LE_OP); }
">="			{ count(); return(GE_OP); }
"=="			{ count(); return(EQ_OP); }
"!="			{ count(); return(NE_OP); }
";"			{ count(); return(';'); }
"{"			{ count(); return('{'); }
"}"			{ count(); return('}'); }
","			{ count(); return(','); }
":"			{ count(); return(':'); }
"="			{ count(); return('='); }
"("			{ count(); return('('); }
")"			{ count(); return(')'); }
"["			{ count(); return('['); }
"]"			{ count(); return(']'); }
"."			{ count(); return('.'); }
"&"			{ count(); return('&'); }
"!"			{ count(); return('!'); }
"~"			{ count(); return('~'); }
"-"			{ count(); return('-'); }
"+"			{ count(); return('+'); }
"*"			{ count(); return('*'); }
"/"			{ count(); return('/'); }
"%"			{ count(); return('%'); }
"<"			{ count(); return('<'); }
">"			{ count(); return('>'); }
"^"			{ count(); return('^'); }
"|"			{ count(); return('|'); }
"?"			{ count(); return('?'); }
"..."                   { count(); return(ELIPSIS); }
#.*\n                   { count(); }

[ \t\v\n\f]		{ count(); }
.			{ /* ignore bad characters */ }

%%

int yywrap()
{
	return(1);
}

void comment()
{
	char c, c1;

loop:
	while ((c = input()) != '*' && c != 0)
		putchar(c);

	if ((c1 = input()) != '/' && c != 0)
	{
		unput(c1);
		goto loop;
	}

	if (c != 0)
		putchar(c1);
}

int column = 0;
int line = 1;

void count()
{
	int i;

	for (i = 0; yytext[i] != '\0'; i++)
		if (yytext[i] == '\n') {
			column = 0;
			++line;
		}
		else if (yytext[i] == '\t')
			column += 8 - (column % 8);
		else
			column++;

//	ECHO;
}

#define NUMTYPES 100000
char types[NUMTYPES] = "";
int  lastType = 0;

void registerTypename(char* nm)
{
	int len = strlen(nm) + 1;
	if (lastType + len >= NUMTYPES) {
		fprintf(stderr,"Yoiks - type table overflow\n");
		exit(1);
	}
	// fprintf(stderr, "Registering '%s'\n", nm);
	strcpy(types+lastType,nm);
	lastType += len;
}

int check_type()
{
	int i = 0;
	//fprintf(stderr, "Searching for '%s'\n", yytext);
	while (i < lastType) {
		if ( strcmp(types+i, yytext) == 0 ) {
			//fprintf(stderr, "Found type %s\n", yytext);
			return(TYPE_NAME);
		}
		//fprintf(stderr, "Didn't match '%s'\n", types+i);
		i += strlen(types+i) + 1;
	}
	//fprintf(stderr, "Didn't find '%s'\n", yytext);
	return(IDENTIFIER);
}

void lex_init()
{
	lastType = 0;
	// registerTypename("__int64_t");
}
