/*
 * Copyright (c) 2000 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * Permission to use, copy, modify, and distribute this file
 * for any purpose with or without restriction is hereby granted.
 */

#include <stdlib.h>

#include "alloc.h"

/*****************************************************************************/

static unsigned int malloc_count;
static unsigned int free_count;

void *
counted_malloc(size_t size)
{
	void *ptr;
	
	ptr = malloc(size);
	if (ptr)
		++malloc_count;
	return ptr;
}

void
counted_free(void *ptr)
{
	if (ptr)
		++free_count;
	free(ptr);
}

/*****************************************************************************/

int
init_alloc_counts(void)
{
	reset_alloc_counts();
	
	/* Successful initialization.  Return 0 to indicate success. */
	return 0;
}

void
get_alloc_counts(unsigned int *mallocs, unsigned int *frees)
{
	*mallocs = malloc_count;
	*frees = free_count;
}

void
reset_alloc_counts(void)
{
	malloc_count = 0;
	free_count = 0;
}

/*****************************************************************************/

/* End of file. */

