/*
 * Copyright (c) 2000 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * Permission to use, copy, modify, and distribute this file
 * for any purpose with or without restriction is hereby granted.
 */
/*
 * XXX --- This file belongs in a separate, common Knit runtime library for
 * user-mode programs.  That runtime library has not yet been implemented.
 */

#include <stdio.h>
#include <stdlib.h>

#include "init.h"

/* Prototypes for the Knit-generated initialization and finalization funcs. */
extern int knit_init(void);
extern int knit_fini(void);

/* Local prototypes. */
static void fini(void);

/*****************************************************************************/

/*
 * `init' invokes the Knit-generated initialization function, and `fini'
 * invokes the Knit-generated finalizer function.
 *
 * `init' and `fini' themselves are invoked automatically when the program
 * starts and finishes, respectively.  We use GCC magic to mark `init', and we
 * use `atexit' to register `fini'.
 *
 * (An alternate implementation would to mark `fini' with the GCC magic
 * `destructor' attribute, but then `fini' would be invoked after `exit', and
 * thus after stdio streams are shut down.  This obviously makes it more
 * difficult to produce error messages for failed finalizers!  Therefore, to
 * keep things simple, we use `atexit' in this implementation.)
 */
static void init(void) __attribute__ ((constructor));

static void
init(void)
{
	int rc;
	
	atexit(&fini);
	rc = knit_init();
	if (rc != 0)
		exit(rc);
}

static void
fini(void)
{
	static int exiting = 0;
	
	/*
	 * The !exiting test is extra protection, e.g., against finalizers that
	 * `panic' and the FreeBSD `exit'/`atexit' implementation which doesn't
	 * guard against multiple calls to `exit'.
	 */
	if (!exiting) {
		exiting = 1;
		knit_fini();
	}
}

/*
 * `knit_progress' is called by the Knit-generated `knit_init_*' functions,
 * after each call to one of our program's initializers and finalizers.
 */
void
knit_progress(const char *func_name, int rc)
{
	/* Knit convention: non-zero `rc' indicates failure. */
	if (rc != 0) {
		fprintf(stderr, "%s failed: returned %d.\n", func_name, rc);
	}
}

/*****************************************************************************/

/* End of file. */

