/*
 * Copyright (c) 2000 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * Permission to use, copy, modify, and distribute this file
 * for any purpose with or without restriction is hereby granted.
 */

#include <stdio.h>
#include <stdlib.h>

#include "common.h"
#include "input.h"

/*****************************************************************************/

static FILE *input_fp = NULL;

int
init_input(void)
{
	input_fp = fdopen(0 /* stdin */, "r");
	if (input_fp == NULL)
		/* Return non-zero to indicate failure. */
		return 1;
	
	/* Successful initialization.  Return 0 to indicate success. */
	return 0;
}

int
fini_input(void)
{
	if (input_fp != NULL) {
		fclose(input_fp);
		input_fp = NULL;
	}
	
	/* Successful finalization.  Return 0 to indicate success. */
	return 0;
}

/*****************************************************************************/

#define BUF_SIZE (256)

static char *
alloc_input(void)
{
	return ((char *) malloc(sizeof(char) * BUF_SIZE));
}

void
free_input(char *str)
{
	free(str);
}

/*****************************************************************************/

int
get_input(char **out_str)
{
	char *buf;
	char *rc;
	
	buf = alloc_input();
	if (!buf)
		return FALSE;
	rc = fgets(buf, BUF_SIZE, input_fp);
	if (rc == NULL) {
		free_input(buf);
		return FALSE;
	}
	*out_str = buf;
	return TRUE;
}

/*****************************************************************************/

/* End of file. */

