/*
 * Copyright (c) 2000 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * Permission to use, copy, modify, and distribute this file
 * for any purpose with or without restriction is hereby granted.
 */

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>

#include "common.h"
#include "number.h"

/*****************************************************************************/

int
string_to_number(const char *str, number *out_num, const char **out_end)
{
	number num;
	char *num_end;
	
	if (!str)
		return FALSE;
	num = strtol(str, &num_end, 10);
	if (num_end == str)
		return FALSE;
	*out_num = num;
	*out_end = num_end;
	return TRUE;
}

int
int_to_number(int i, number *out_num)
{
	*out_num = i;
	return TRUE;
}

int
copy_number(number n, number *out_num)
{
	*out_num = n;
	return TRUE;
}

void
free_number(number n)
{
	/* Nothing to do. */
}

/*****************************************************************************/

int
number_is_zero(number n)
{
	return (n == 0);
}

int
number_neg(number n, number *out)
{
	*out = -n;
	return TRUE;
}

int
number_add(number a, number b, number *out)
{
	*out = a + b;
	return TRUE;
}

int
number_sub(number a, number b, number *out)
{
	*out = a - b;
	return TRUE;
}

int
number_mul(number a, number b, number *out)
{
	*out = a * b;
	return TRUE;
}

int
number_div(number a, number b, number *out)
{
	if (!b)
		return FALSE;
	*out = a / b;
	return TRUE;
}

/*****************************************************************************/

void
number_print(number n)
{
	printf("%ld", n);
}

/*****************************************************************************/

/* End of file. */

