;; knit-mode.el --- Major mode for editing Knit code.
;;
;; Copyright (c) 2000, 2001 University of Utah and the Flux Group.
;; All rights reserved.
;; 
;; This file is part of the Knit component composition software.
;; 
;; Knit is free software, also known as "open source."  You can redistribute
;; it and/or modify it under the terms of the GNU General Public License
;; (GPL), version 2, as published by the Free Software Foundation (FSF).
;; 
;; Knit is distributed in the hope that it will be useful, but WITHOUT ANY
;; WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
;; received a copy of the GPL along with Knit; see the file COPYING.  If
;; not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
;; 
;; Users are requested, but not required, to send to csl-dist@cs.utah.edu any
;; improvements that they make and grant Univ. of Utah redistribution rights.
;;

;; This is a very quick, very first-pass hack.
;; Bug reports to Eric Eide <eeide@cs.utah.edu>.

(require 'cc-mode)
(require 'font-lock)

;;;; *** `cc-langs.el' stuff.

(defconst c-knit-class-key c-C-class-key) ;; XXX --- Bogus.
(defconst c-knit-baseclass-key nil)
;; putting flags in here goofs up indentation of `flags' sections within units.
;;(defconst c-knit-extra-toplevel-key "\\(bundletype\\|flags\\|package\\|unit\\)")
(defconst c-knit-extra-toplevel-key "\\(package\\|unit\\)")
(defconst c-knit-access-key nil)
(defconst c-knit-conditional-key c-C-conditional-key)
(defconst c-knit-method-key nil)
(defconst c-knit-comment-start-regexp c-C++-comment-start-regexp)

;;;; *** `cc-vars.el' stuff.

(defcustom knit-mode-hook nil
  "*Hook called by `knit-mode'."
  :type 'hook
  :group 'c)

(setq auto-mode-alist (append '(("\\.unit\\'" . knit-mode))
			      auto-mode-alist))

;;;; **** `cc-langs.el' stuff.

(defvar c-knit-menu nil)

;; Support for Knit.

(defvar knit-mode-abbrev-table nil
  "Abbreviation table used in knit-mode buffers.")
(define-abbrev-table 'knit-mode-abbrev-table ())

(defvar knit-mode-map ()
  "Keymap used in knit-mode buffers.")
(if knit-mode-map
    nil
  (setq knit-mode-map (c-make-inherited-keymap))
  ;; add bindings which are only useful for Knit.
  (define-key knit-mode-map "\C-c\C-e" 'c-macro-expand)
  (define-key knit-mode-map "\C-c:"    'c-scope-operator)
  )

;;;###autoload
(defvar knit-mode-syntax-table nil
  "Syntax table used in knit-mode buffers.")
(if knit-mode-syntax-table
    ()
  (setq knit-mode-syntax-table (make-syntax-table))
  (c-populate-syntax-table knit-mode-syntax-table)
  ;; Quote is sometimes used to mean ``prime.''
  (modify-syntax-entry ?' "_" knit-mode-syntax-table)
  ;; TBD: does it make sense for colon to be symbol class in Knit?
  ;; I'm not so sure, since c-label-key is busted on lines like:
  ;; Foo::bar( i );
  ;; maybe c-label-key should be fixed instead of commenting this out,
  ;; but it also bothers me that this only seems appropriate for Knit/C++
  ;; and not C.
  ;;(modify-syntax-entry ?: "_" knit-mode-syntax-table)
  )

(defvar cc-imenu-knit-generic-expression
  cc-imenu-c-generic-expression
  "Imenu generic expression for Knit mode.  See `imenu-generic-expression'."
  )

(easy-menu-define c-knit-menu knit-mode-map "Knit Mode Commands"
		  (c-mode-menu "Knit"))


;;;; *** `cc-mode.el' stuff.

;;;###autoload
(defun knit-mode ()
  "Major mode for editing Knit code.

The hook variable `knit-mode-hook' is run with no args, if that value is
bound and has a non-nil value.  Also the hook `c-mode-common-hook' is
run first.

Key bindings:
\\{knit-mode-map}"
  (interactive)
  (c-initialize-cc-mode)
  (kill-all-local-variables)
  (set-syntax-table knit-mode-syntax-table)
  (setq major-mode 'knit-mode
	mode-name "Knit"
	local-abbrev-table knit-mode-abbrev-table)
  (use-local-map knit-mode-map)
  (c-common-init)
  (setq comment-start			"/* "
	comment-end			" */"
	c-recognize-knr-p		nil
	
	c-conditional-key		c-knit-conditional-key
	c-class-key			c-knit-class-key
	c-method-key			c-knit-method-key
	c-access-key			c-knit-access-key
	c-baseclass-key			c-knit-baseclass-key
	c-extra-toplevel-key		c-knit-extra-toplevel-key
	c-comment-start-regexp		c-knit-comment-start-regexp
	
	imenu-generic-expression	cc-imenu-knit-generic-expression
	imenu-case-fold-search		nil
	)
  (run-hooks 'c-mode-common-hook)
  (run-hooks 'knit-mode-hook)
  (c-update-modeline))

;;;; *** `font-lock.el' stuff.

(put 'knit-mode 'font-lock-defaults 
     '((knit-font-lock-keywords
	knit-font-lock-keywords-1
	knit-font-lock-keywords-2
	knit-font-lock-keywords-3)
       nil
       nil
       ((?_ . "w"))
       beginning-of-defun)
     )

(defconst knit-font-lock-keywords-1 nil
  "Subdued level highlighting for Knit mode.")

(defconst knit-font-lock-keywords-2 nil
  "Medium level highlighting for Knit mode.")

(defconst knit-font-lock-keywords-3 nil
  "Gaudy level highlighting for Knit mode.")

(let ((knit-token-regexp
       "\\(\\sw\\|\\s_\\)+")
      (knit-keyword-regexp
       (concat "\\<\\("
	       (mapconcat 'identity
			  '(
			    ;; List made by searching through `ParseUnits.hs'
			    ;; for `lit "..."'.
			    "adding"
			    "bundletype"
			    "constraints"
			    "default"
			    "defaults"
			    "deleting"
			    "depends"
			    "directory"
			    "export"
			    "exports"
			    "extends"
			    "files"
			    "finalizer"
			    "find"
			    "fini"
			    "finis"
			    "flags"
			    "flatten"
			    "for"
			    "global"
			    "glue"
			    "import"
			    "imports"
			    "include"
			    "init"
			    "initializer"
			    "inits"
			    "link"
			    "modify"
			    "needs"
			    "noflatten"
			    "package"
			    "prefix"
			    "property"
			    "rename"
			    "rule"
			    "rules"
			    "suffix"
			    "to"
			    "type"
			    "unit"
			    "var"
			    "with"
			    )
			  "\\|")
	       "\\)\\>")
       ))
  (setq knit-font-lock-keywords-1
	(append
	 c-font-lock-keywords-1
	 (list
	  (list knit-keyword-regexp 1 'font-lock-keyword-face)
	  ;;
	  (list (concat
		 "^\\(bundletype\\|flags\\|package\\|property\\|rule\\|type\\|unit\\)"
		 "[ \t]+"
		 "\\(" knit-token-regexp "\\)"
		 )
		2
		;; XXX --- Well, clearly, none of the above keywords introduce
		;; functions!
		'font-lock-function-name-face
		)
	  )
	 ))
  (setq knit-font-lock-keywords-2
	(append
	 knit-font-lock-keywords-1
	 ;; More patterns here.
	 ))
  (setq knit-font-lock-keywords-3
	(append
	 knit-font-lock-keywords-2
	 ;; More patterns here.
	 ))
  )

(defvar knit-font-lock-keywords knit-font-lock-keywords-1
  "Default expressions to highlight in Knit mode.")


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; End of file.

